from typing import Any, Dict

from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.compiler import SQLCompiler
from sqlalchemy.sql.expression import FunctionElement
from sqlalchemy.types import DateTime


class utc_now(FunctionElement):
    """Current timestamp in UTC timezone"""

    type = DateTime()


@compiles(utc_now, "postgresql")
def pg(element: utc_now, compiler: SQLCompiler, **kw: Dict[str, Any]) -> SQLCompiler:
    return "timezone('utc', current_timestamp)"


@compiles(utc_now, "sqlite")
def sqlite(
    element: utc_now, compiler: SQLCompiler, **kw: Dict[str, Any]
) -> SQLCompiler:
    return "datetime('now')"


@compiles(utc_now, "mysql")
def mysql(element: utc_now, compiler: SQLCompiler, **kw: Dict[str, Any]) -> SQLCompiler:
    return "utc_timestamp()"



from typing import Any, Dict

from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.compiler import SQLCompiler
from sqlalchemy.sql.expression import FunctionElement
from sqlalchemy.types import JSON


class agg(FunctionElement):
    """JSON array aggregation"""

    type = JSON()
    name = "agg"


@compiles(agg, "postgresql")
def pg(element: agg, compiler: SQLCompiler, **kw: Dict[str, Any]) -> SQLCompiler:
    return "jsonb_agg(%s)" % compiler.process(element.clauses, **kw)


@compiles(agg, "sqlite")
def sqlite(element: agg, compiler: SQLCompiler, **kw: Dict[str, Any]) -> SQLCompiler:
    return "json_group_array(%s)" % compiler.process(element.clauses, **kw)


@compiles(agg, "mysql")
def mysql(element: agg, compiler: SQLCompiler, **kw: Dict[str, Any]) -> SQLCompiler:
    return "json_arrayagg(%s)" % compiler.process(element.clauses, **kw)



from typing import Any, Dict

from sqlalchemy import func as sqla_func
from sqlalchemy import select
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql import cast
from sqlalchemy.sql.compiler import SQLCompiler
from sqlalchemy.sql.expression import FunctionElement
from sqlalchemy.sql.type_api import TypeEngine


class to_array(FunctionElement):
    """JSON object creation"""

    name = "to_array"


function = to_array


@compiles(function, "postgresql")
def pg(element: function, compiler: SQLCompiler, **kw: Dict[str, Any]) -> SQLCompiler:

    args = element.clauses

    if len(args) != 2:
        raise Exception("Invalid n arguments for to_array")

    json_field, type_bind_param = args

    type_ = type_bind_param.value

    assert isinstance(type_, TypeEngine)

    select_from = sqla_func.jsonb_array_elements(cast(json_field, JSONB())).alias()
    statement = (
        select([sqla_func.array_agg(cast(select_from.column, type_))])
        .select_from(select_from)
        .scalar_subquery()
    )

    return compiler.process(statement, **kw)



from typing import Any, Dict

from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.compiler import SQLCompiler
from sqlalchemy.sql.expression import FunctionElement
from sqlalchemy.types import JSON


class build_object(FunctionElement):
    """JSON object creation"""

    type = JSON()
    name = "jsonb_build_object"


@compiles(build_object, "postgresql")
def pg_build_object(
    element: build_object, compiler: SQLCompiler, **kw: Dict[str, Any]
) -> SQLCompiler:
    return "jsonb_build_object(%s)" % compiler.process(element.clauses, **kw)


@compiles(build_object, "sqlite")
@compiles(build_object, "mysql")
def other_build_object(
    element: build_object, compiler: SQLCompiler, **kw: Dict[str, Any]
) -> SQLCompiler:
    return "json_object(%s)" % compiler.process(element.clauses, **kw)
